from manga_py.fs import get_temp_path, rename, path_join, dirname
from manga_py.provider import Provider
from .helpers.std import Std


# Archive downloading example. Without images
class MangaOnlineBiz(Provider, Std):
    chapter_url = ''

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/download/[^/]+/.+?_(\d+)_(\d+)', self.chapter_url).groups()
        return '{}-{}'.format(*idx)

    def get_main_content(self):
        return self._get_content('{}/{}.html')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.biz/([^/]+)(?:/|\.html)')

    def get_arc_path(self):
        path = self.get_archive_name()
        name = dirname(self.get_archive_path())
        arc = self._archive_type()
        return path_join(path, name + '.%s' % arc)

    def download_volume(self, idx, url, manga_name):
        temp_path = get_temp_path('{:0>2}_{}-temp_arc.zip'.format(idx, manga_name))
        self.save_file(self.http().normalize_uri(url), temp_path)
        rename(temp_path, self.get_arc_path())

    def loop_chapters(self):
        _min = self._params.get('skip_volumes', 0)
        _max = _min + self._params.get('max_volumes', 0)
        volumes = self._storage['chapters']
        for idx, url in enumerate(volumes):
            if idx < _min or (idx > _max > 0) or self._check_archive():
                continue
            self.chapter_url = url
            self.download_volume(idx, url, self.manga_name)

    def get_chapters(self):
        s, c = r'MangaChapter\((.+)\);', self.content
        items = self.json.loads(self.re.search(s, c).group(1))
        n = self.http().normalize_uri
        return [n(i.get('downloadUrl')) for i in items]

    def get_files(self):
        return []

    def get_cover(self):
        return self._cover_from_content('.item > .image > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaOnlineBiz



from .somanga_net import SoMangaNet


class UnionMangasNet(SoMangaNet):

    def get_chapters(self):
        selector = '.tamanho-bloco-perfil .lancamento-linha a[href*="/leitor/"]'
        return self.document_fromstring(self.content, selector)


main = UnionMangasNet



import base64
import codecs
import gzip
import zlib
from binascii import unhexlify
from struct import pack, unpack

from Crypto.Cipher import AES
from Crypto.Hash import SHA256, MD5
from execjs import compile


class BaseLib:

    @staticmethod
    def decode_escape(data):  # pragma: no cover

        if isinstance(data, str):
            data = data.encode()
        try:
            data = codecs.escape_decode(data)
            return data[0]
        except Exception:
            return ''

    @staticmethod
    def encode_hex(data):  # pragma: no cover
        return codecs.decode(data, 'hex')

    @staticmethod
    def to_sha_256(data):  # pragma: no cover
        if isinstance(data, str):
            data = data.encode()
        sha = SHA256.new()
        sha.update(data)
        return sha.digest()

    @staticmethod
    def decrypt_aes(iv, key, data):  # pragma: no cover
        aes = AES.new(key, AES.MODE_CBC, iv)
        return aes.decrypt(data)

    @staticmethod
    def base64decode(data, altchars=None, validate=False):  # pragma: no cover
        return base64.b64decode(data, altchars, validate)

    @staticmethod
    def base64encode(data, altchars=None):  # pragma: no cover
        return base64.b64encode(data, altchars)

    @staticmethod
    def exec_js(source, js):  # pragma: no cover
        return compile(source).eval(js)

    @staticmethod
    def gunzip(data):  # pragma: no cover
        return gzip.decompress(data)

    @staticmethod
    def gzip(data, lvl: int = 9):  # pragma: no cover
        return gzip.compress(data, lvl)

    @staticmethod
    def zlib_d(data, **kwargs):  # pragma: no cover
        return zlib.decompress(data, **kwargs)

    @staticmethod
    def zlib_c(data, **kwargs):  # pragma: no cover
        return zlib.compress(data, **kwargs)

    @staticmethod
    def md5(string):  # pragma: no cover
        _ = MD5.new()
        _.update(string.encode())
        return _

    @staticmethod
    def pack(fmt, *args):  # pragma: no cover
        return pack(fmt, *args)

    @staticmethod
    def unpack(fmt, string):  # pragma: no cover
        return unpack(fmt, string)

    @staticmethod
    def pack_auto(int_list) -> bytes:
        """
        :param int_list: list
        :return: str
        """
        base_frm = '{}B'.format(len(int_list))
        return pack(base_frm, *int_list)

    @staticmethod
    def unpack_auto(string) -> list:
        """
        :param string: str
        :return: tuple
        """
        if isinstance(string, str):
            string = string.encode()

        return list(string)

    @staticmethod
    def str2hex(string):
        hex_str = ''
        if isinstance(string, bytes):
            string = string.decode()
        for char in string:
            int_char = ord(char)
            hex_num = hex(int_char).lstrip("0x")
            hex_str += hex_num
        return hex_str

    @staticmethod
    def hex2str(string):
        clear_str = ''
        if isinstance(string, bytes):
            string = string.decode()
        for counter in range(0, len(string), 2):
            hex_char = string[counter] + string[counter + 1]
            clear_str += unhexlify(hex_char)
        return clear_str



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicOnlineTo(Provider, Std):
    def get_archive_name(self) -> str:
        chapter = self.re.search('id=(\d+)', self.chapter).group(1)
        return self.normal_arc_name([self.chapter_id, chapter])

    def get_chapter_index(self, no_increment=False) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        return self._get_content('{}/Comic/{}')

    def get_manga_name(self) -> str:
        return self.re.search(r'\.to/Comic/([^/]+)', self.get_url())

    def get_chapters(self):
        return self.document_fromstring(self.content, 'table.listing td > a')

    def prepare_cookies(self):
        self.cf_protect(self.get_url())
        self._storage['cookies']['rco_quality'] = 'hq'

    def get_files(self):
        content = self.http_get(self.chapter + '&readType=1')
        items = self.re.findall('lstImages.push\("([^"]+)"\)', content)
        return items

    def get_cover(self):
        return self._cover_from_content('.rightBox .barContent img[width]')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadComicOnlineTo



from .helpers.std import Std
from .readhentaimanga_com import ReadHentaiMangaCom


class HentaiReadCom(ReadHentaiMangaCom, Std):

    def get_chapters(self):
        return self._elements('.read-now a.lst')


main = HentaiReadCom



from .gomanga_co import GoMangaCo
from .helpers.std import Std


class WebtoonTrCom(GoMangaCo, Std):
    _name_re = '/_/([^/]+)'
    _content_str = '{}/_/{}'
    _chapters_selector = '.panel-default table td > a'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('.', 2)
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        url = self.chapter
        index_re = r'/_/[^/]+/(.+)'
        return self.re.search(index_re, url).group(1)

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.cImg')

    def get_cover(self) -> str:
        return self._cover_from_content('.left img.image')


main = WebtoonTrCom




class Http2:
    provider = None
    path_join = None
    is_file = None

    def __init__(self, provider):
        from manga_py.fs import path_join, is_file
        self.provider = provider
        self.path_join = path_join
        self.is_file = is_file

    def _get_name(self, url):
        archive = self.provider._params.get('name', '')
        if not len(archive):
            archive = self.provider._storage['manga_name']
        return self.path_join(
            self.provider._params.get('destination', 'Manga'),
            archive,
            url[0]
        )

    def download_archives(self):
        volumes = self.provider._storage['chapters']
        _min = self.provider._params.get('skip_volumes', 0)
        _max = _min + self.provider._params.get('max_volumes', 0)
        for idx, url in enumerate(volumes):
            self.provider._storage['current_chapter'] = idx
            name = self._get_name(url)
            if idx < _min or (idx > _max > 0) or self.is_file(name):
                continue
            self.provider.http().download_file(url[1], name)



from manga_py.provider import Provider
from .helpers.std import Std


class ReadmangaMe(Provider, Std):

    def __current_ch(self):
        _re = '/.+/(?:vol)?([^/]+/[^/]+)(?:/|\?ma?t)?'
        name = self.re.search(_re, self.chapter).group(1)
        if ~name.find('?'):
            name = name[:name.find('?')]
        return name

    def get_archive_name(self):
        name = self.__current_ch()
        return self.normal_arc_name(name.split('/'))

    def get_chapter_index(self):
        name = self.__current_ch()
        return name.replace('/', '-')

    def get_main_content(self):
        return self._get_content('{}/{}?mature=1&mtr=1')

    def get_manga_name(self):
        return self._get_name(r'\.me/([^/]+)')

    def get_chapters(self):
        return self._elements('div.chapters-link tr > td > a')

    def get_files(self):
        _uri = self.http().normalize_uri(self.chapter)
        content = self.http_get(_uri)
        result = self.re.search(r'rm_h\.init.+?(\[\[.+\]\])', content, self.re.M)
        if not result:
            return []
        return [i[1] + i[0] + i[2] for i in self.json.loads(result.groups()[0].replace("'", '"'))]

    def get_cover(self):
        return self._cover_from_content('.picture-fotorama > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadmangaMe



import requests
from lxml.html import document_fromstring


class AutoProxy:
    checked_url = 'https://httpbin.org/ip'

    @staticmethod
    def __strip(text):
        return text.text_content().strip(' \n\t\r\0')

    def _s(self, item):
        td = item.cssselect('td')
        proxy = self.__strip(td[4])  # proxy type
        https = self.__strip(td[6])  # https (yes|no)
        if (
                proxy == 'anonymous'
                or proxy == 'elite proxy'
        ) and https == 'yes':
            return self.__strip(td[0]) + ':' + self.__strip(td[1])
        return None

    def _test_proxy(self, url):
        proxies = {
            'http': url,
            'https': url,
        }
        try:
            requests.head(url=self.checked_url, proxies=proxies, timeout=6)
        except Exception:
            return False
        return proxies

    def _change_checked_url(self, checked_url):
        if checked_url:
            self.checked_url = checked_url

    def auto_proxy(self, checked_url=None):
        self._change_checked_url(checked_url)
        url = 'https://www.us-proxy.org'
        items = document_fromstring(requests.get(url).text)
        items = items.cssselect('#proxylisttable tbody tr')
        for n, i in enumerate(items):
            proxy = self._s(i)
            test = False
            if proxy:
                test = self._test_proxy(proxy)
            if test:
                return test
        return None


auto_proxy = AutoProxy().auto_proxy



from shutil import copy

from manga_py.fs import is_file, storage, root_path, path_join
from .mangachan_me import MangaChanMe


class HentaiChanMe(MangaChanMe):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def _login(self, **kwargs):
        url = self.domain + '/index.php'
        login = kwargs.get('login', '')
        password = kwargs.get('password', '')
        method = kwargs.get('method', 'post')
        data = {'login_name': login, 'login_password': password, 'image': 'Вход', 'login': 'submit'}
        response = self.http().requests(method=method, data=data, url=url)
        cookies = {}
        for i in response.cookies.items():
            cookies[i[0]] = i[1]
        return cookies

    def prepare_cookies(self):
        _storage = storage('.passwords.json')
        if not is_file(_storage):
            copy(path_join(root_path(), 'manga_py', '.passwords.json.dist'), _storage)
        file = open(_storage, 'r').read()
        data = self.json.loads(file).get('hentai_chan_me', {})
        cookies = self._login(**data)
        for i in cookies:
            self._storage['cookies'][i] = cookies[i]

    def get_chapters(self):
        name = self.re.search(self._full_name_selector, self.get_url())
        url = '{}/related/{}'.format(self.domain, name.group(1))
        chapters = self.html_fromstring(url, '.related .related_info > h2 a')
        nu = self.http().normalize_uri
        return [nu(i.get('href').replace('/manga/', '/online/')) for i in chapters]

    def book_meta(self) -> dict:
        # todo meta
        pass


main = HentaiChanMe



from manga_py.provider import Provider
from .helpers.std import Std


class TuMangaOnlineCom(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.chapter)

    def get_chapter_index(self) -> str:
        return '{}-{}'.format(*self.chapter)

    def get_main_content(self):
        url = '{}/api/v1/mangas/{}'.format(self.domain, self._get_id())
        data = self.http_get(url=url, headers=self._get_headers())
        return self.json.loads(data)

    def get_manga_name(self) -> str:
        url = self.get_url()
        re = r'/mangas/\d+/([^/]+)'
        if ~url.find('/lector/'):
            re = '/lector/([^/]+)'
        return self.re.search(re, url).group(1)

    @staticmethod
    def _get_subidas(items, n):
        return [(n, i.get('idScan')) for i in items.get('subidas', [])]

    def get_chapters(self):
        url = '{}/api/v1/mangas/{}/capitulos?page=1&tomo=-1'
        idx = self._get_id()

        url = url.format(self.domain, idx)
        data = self.http_get(url=url, headers=self._get_headers())
        items = self.json.loads(data).get('data', [])

        pages = []
        for i in items:
            n = i.get('numCapitulo')
            pages += self._get_subidas(i, n)
        return pages

    def _get_id(self):
        url = self.get_url()
        re = r'/mangas/(\d+)'
        if ~url.find('/lector/'):
            re = r'/lector/[^/]+/(\d+)'
        return self.re.search(re, url).group(1)

    @staticmethod
    def _get_headers():
        return {'Cache-mode': 'no-cache', 'X-Requested-With': 'XMLHttpRequest'}

    def prepare_cookies(self):
        self._base_cookies()

    def get_files(self):
        idx = self._get_id()
        domain = self.domain
        url = '{}/api/v1/imagenes?idManga={}&idScanlation={}&numeroCapitulo={}&visto=true'
        ch = self.chapter
        data = self.http_get(url=url.format(domain, idx, *ch), headers=self._get_headers())

        items = self.json.loads(self.json.loads(data).get('imagenes', '[]'))
        url = 'https://img1.tumangaonline.com/subidas/{}/{}/{}/{}'

        return [url.format(idx, *ch, i) for i in items]

    def get_cover(self) -> str:
        url = self.domain, self.content.get('imageUrl', None)
        if url:
            return '{}/{}'.format(self.domain, url)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TuMangaOnlineCom



from os import path

from PIL import Image as PilImage, ImageChops


class Image:
    image = None
    src_path = None

    def __init__(self, src_path):
        """
        :param src_path:
        """
        if not path.isfile(src_path):
            raise AttributeError('Image not found')

        self.src_path = src_path
        self.__open(src_path)

    def __open(self, _path):
        """
        :param _path:
        :return:
        """
        if self.image is None:
            self.image = PilImage.open(_path)

    def gray(self, dest_path: str):
        """
        :param dest_path:
        :return:
        """
        try:
            image = self.image.convert('LA')
        except (ValueError, OSError):
            image = self.image.convert('L')
        if dest_path is not None:
            image.save(dest_path)
        return image

    def convert(self, dest_path: str, quality: int = 95):
        """
        see http://pillow.readthedocs.io/en/3.4.x/handbook/image-file-formats.html
        :param dest_path:
        :param quality:
        :return:
        """
        self.image.save(dest_path, quality=quality)
        return dest_path

    def crop_manual_with_offsets(self, offsets, dest_path: str):
        """
        :param offsets:
        :param dest_path:
        :return:
        """
        left, upper, right, lower = offsets
        width, height = self.image.size
        image = self.image.crop((
            left,
            upper,
            width - right,
            height - lower
        ))
        image.save(dest_path)

    def crop_manual(self, sizes: tuple, dest_path: str):
        """
        :param sizes: The crop rectangle, as a (left, upper, right, lower)-tuple.
        :param dest_path:
        :return:
        """
        self.image.crop(sizes).save(dest_path)

    def crop_auto(self, dest_path: str):
        """
        :param dest_path:
        :return:
        """
        bg = PilImage.new(
            self.image.mode,
            self.image.size,
            self.image.getpixel((0, 0))
        )
        diff = ImageChops.difference(self.image, bg)
        diff = ImageChops.add(diff, diff, 2.0, -100)
        bbox = diff.getbbox()
        if bbox:
            crop = self.image.crop(bbox)
            if dest_path:
                crop.save(dest_path)

    def close(self):
        self.image is not None and self.image.close()



from manga_py.provider import Provider
from .helpers.std import Std


class MangaDexCom(Provider, Std):
    _links_on_page = 100
    _home_url = ''

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        ch = self.chapter[0]
        re = self.re.compile(r'[vV]ol.+?(\d+).+?[cC]h.+?(\d+(?:.\d+)?)')
        return '{}-{}'.format(*self._idx_to_x2(re.search(ch).groups()))

    def get_main_content(self):
        url = self.get_url()
        if url.find('/manga/') < 0:
            url = self.html_fromstring(url, '.toggle.col-sm-2 > a', 0)
            url = self.http().normalize_uri(url.get('href'))
        return self.http_get(url)

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/manga/'):
            name = self.html_fromstring(url, 'h3.panel-title', 0)
        else:
            name = self.html_fromstring(url, '.toggle.col-sm-2 > a', 0)
        return name.text_content().strip()

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        pages = self._get_pages_count(parser)
        items = self._chapters(parser)
        for i in range(pages):
            n = i * self._links_on_page
            parser = self.html_fromstring('%s/%d' % (self._links_on_page, n))
            items += self._chapters(parser)
        return self._parse_chapters(items)

    def get_files(self):
        content = self.http_get(self.chapter[1])
        items = self.re.search(r'page_array.+\n?(.+),]', content)
        data = self.re.search(r'dataurl\s*=\s*[\'"](.+)[\'"]', content)
        server = self.re.search(r'server\s*=\s*[\'"](.+)[\'"]', content)
        if not items or not data or not server:
            return []
        items = items.group(1).strip(' \'').split('\',\'')
        data = data.group(1)
        server = server.group(1)
        domain = self.domain
        return ['{}{}{}/{}'.format(domain, server, data, i) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.edit .col-sm-3 img')

    def prepare_cookies(self):
        self._storage['cookies']['mangadex_h_toggle'] = '1'

    def _parse_chapters(self, items):
        n = self.http().normalize_uri
        return [(i.text_content(), n(i.get('href'))) for i in items]

    @staticmethod
    def _chapters(parser):
        return parser.cssselect('#chapters tr td:first-child a')

    def _get_pages_count(self, parser):
        pages = parser.cssselect('.pagination .paging > a')
        count = 0
        if pages:
            re = self.re.compile(r'.+/(\d+)')
            count = re.search(pages[-1].get('href')).group(1)
        return int(count / self._links_on_page)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaDexCom



from urllib.parse import urlparse


class UrlNormalizer:

    @staticmethod
    def _parse_sheme(parse, base_parse):
        if not parse.scheme:
            uri = base_parse.scheme
        else:
            uri = parse.scheme
        return uri + '://'

    @staticmethod
    def _parse_netloc(parse, base_parse):
        if not parse.netloc:
            uri = base_parse.netloc
        else:
            uri = parse.netloc
        return uri

    @staticmethod
    def _test_path_netloc(parse):
        if parse.path.find('://') == 0:
            return urlparse('http' + parse.path).path
        return parse.path

    @staticmethod
    def __parse_rel_path(parse, base_parse):
        path = ''
        if base_parse.path.rfind('/') > 0:
            path = base_parse.path[0:base_parse.path.rfind('/')]
        return path.rstrip('/') + '/' + parse.path.lstrip('/')

    @staticmethod
    def _parse_path(parse, base_parse):
        _path = UrlNormalizer._test_path_netloc(parse)
        if _path:
            if _path.find('/') == 0:
                return _path
            else:
                return UrlNormalizer.__parse_rel_path(parse, base_parse)
        else:
            return base_parse.path

    @staticmethod
    def _parse_query(parse):
        if parse.query:
            return '?' + parse.query
        return ''

    @staticmethod
    def _parse_fragment(parse):
        if parse.fragment:
            return '#' + parse.fragment
        return ''

    @staticmethod
    def url_helper(url: str, base_url: str) -> str:
        parse = urlparse(url)
        base_parse = urlparse(base_url)
        un = UrlNormalizer
        sheme = un._parse_sheme(parse, base_parse)
        netloc = un._parse_netloc(parse, base_parse)
        path = un._parse_path(parse, base_parse)
        query = un._parse_query(parse)
        fragment = un._parse_fragment(parse)
        return sheme + netloc + path + query + fragment


normalize_uri = UrlNormalizer.url_helper



from .zeroscans_com import ZeroScansCom


class TrashScanlationsCom(ZeroScansCom):
    def get_main_content(self):
        return self._get_content('{}/series/{}/')

    def get_manga_name(self) -> str:
        return self._get_name('/series/([^/]+)')


main = TrashScanlationsCom



import urllib3

from manga_py.provider import Provider
from .helpers.std import Std


class MangaTownCom(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search('/manga/[^/]+/c([^/]+)', self.chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_main_content(self):
        return self._get_content('{}/manga/{}/')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)/?')

    def get_chapters(self):
        return self.document_fromstring(self.content, '.chapter_list a')

    def prepare_cookies(self):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self._http_kwargs['verify'] = False

    def get_files(self):
        img_selector = 'img#image'
        url = self.http().normalize_uri(self.chapter)
        parser = self.html_fromstring(url)
        selector = '#top_chapter_list + .page_select select option + option'
        images = self._images_helper(parser, img_selector)

        for i in parser.cssselect(selector):
            url = self.http().normalize_uri(i.get('value'))
            img = self.html_fromstring(url)
            images += self._images_helper(img, img_selector)

        return images

    def get_cover(self):
        return self._cover_from_content('.detail_info > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaTownCom



from time import time
from urllib import parse

from manga_py.crypt.puzzle import Puzzle
from manga_py.fs import move
from manga_py.provider import Provider
from .helpers.std import Std


class PlusComicoJp(Provider, Std):
    scrambles = []

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return self.re.search('/store/\d+/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content:
            return content
        idx = self.re.search('/store/(\d+)', self.get_url())
        url = '{}/store/{}/'.format(self.domain, idx.group(1))
        return self.http_get(url)

    def get_manga_name(self) -> str:
        return self.text_content(self.content, 'h1 > ._title')

    def get_chapters(self):
        idx = self.re.search(r'/store/(\d+)', self.get_url()).group(1)
        data = self.http_post('{}/store/api/getTitleArticles.nhn'.format(
            self.domain
        ), data={
            'titleNo': idx
        })
        json = self.json.loads(data)
        items = []
        for i in json.get('result', {}).get('list', {}):
            for m in i.get('articleList'):
                if m.get('freeFlg') == 'Y':
                    items.append(m.get('articleDetailUrl'))
        return items

    def get_files(self):
        url = self.http().requests(self.chapter, method='head')
        location = url.headers.get('location')
        self.http().requests(location, method='head')

        location = parse.urlparse(location)
        params = parse.parse_qs(location.query)

        ts = int(time())
        base_url = '{}://{}{}/diazepam_hybrid.php?param={}&ts={}&_={}&reqtype=0'.format(
            location.scheme,
            location.netloc,
            self.re.search(r'(.+)/\w+\.php', location.path).group(1),
            parse.quote_plus(params.get('param')[0]),
            ts,
            ts + 1305,
        )

        pages_url = base_url + '&mode=7&file=face.xml&callback=jQ12_34'
        scramble_url = base_url + '&mode=8&file={:0>4}.xml'
        file_url = base_url + '&mode=1&file={:0>4}_0000.bin'

        total_pages = self.re.search(r'TotalPage>(\d+)</TotalPage', self.http_get(pages_url))
        if total_pages:
            total_pages = int(total_pages.group(1))
        else:
            total_pages = 0
        items = []
        self.scrambles = []
        for i in range(total_pages):
            c = self.re.search(r'Scramble>(.+?)</Scramble', self.http_get(scramble_url.format(i)))
            self.scrambles.append(c.group(1))
            items.append(file_url.format(i))
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img')

    def after_file_save(self, _path: str, idx: int):
        _matrix = self.scrambles[idx].split(',')
        div_num = 4
        matrix = {}
        n = 0
        for i in _matrix:
            matrix[int(i)] = n
            n += 1
        p = Puzzle(div_num, div_num, matrix, 8)
        p.need_copy_orig = True
        p.de_scramble(_path, '{}.jpg'.format(_path))
        move('{}.jpg'.format(_path), _path)

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = '{}_image.jpg'.format(idx)
        super().save_file(idx, callback, url, in_arc_name)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PlusComicoJp



from manga_py.provider import Provider
from .helpers.std import Std


class MangaLibMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        selector = r'\.me/[^/]+/[^\d]+(\d+)/[^\d]+([^/]+)'
        idx = self.re.search(selector, self.chapter).groups()
        return '-'.join(idx)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.me/([^/]+)')

    def get_chapters(self):
        return self._elements('.chapters-list .chapter-item__name a')

    def get_files(self):
        content = self.http_get(self.chapter)
        base_url = self.re.search(r'\bimgUrl: *[\'"]([^\'"]+)', content).group(1)
        images = self.re.search(r'\bpages: *(\[\{.+\}\])', content).group(1)
        images = self.json.loads(images)
        imgs = ['https://img2.mangalib.me{}{}'.format(
            base_url,
            i.get('page_image'),
        ) for i in images]
        return imgs

    def get_cover(self):
        return self._cover_from_content('img.manga__cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaLibMe



from manga_py.provider import Provider
from .helpers.std import Std


class WieMangaCom(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name([
            self.chapter_id,
            self.re.search('/chapter/([^/]+)', self.chapter).group(1)
        ])

    def get_chapter_index(self) -> str:
        return self.re.search(r'/chapter/[^/]+/(\d+)/', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}.html')

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/chapter/'):
            url = self.html_fromstring(url, '.sitemaplist a + a', 0).get('href')
            url = self.http().normalize_uri(url)
        return self.re.search(r'/manga/([^/]+)\.html', url).group(1)

    def get_chapters(self):
        return self._elements('.chapterlist .col1 a')

    def get_files(self):
        selector = 'img#comicpic'
        parser = self.html_fromstring(self.chapter)
        pages = self._first_select_options(parser, 'select#page')
        items = self._images_helper(parser, selector)
        for i in pages:
            parser = self.html_fromstring(i.get('value'))
            items += self._images_helper(parser, selector)
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('.bookfrontpage a > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = WieMangaCom



from lxml.html import HtmlElement

from manga_py.provider import Provider
from .helpers import e_hentai_org
from .helpers.std import Std


class EHentaiOrg(Provider, Std):
    helper = None

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        url = None
        if isinstance(url, HtmlElement):
            url = self.helper.get_image(url)
        return super().save_file(idx=idx, callback=callback, url=url, in_arc_name=in_arc_name)

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        return self.http_get(self.helper.get_url())

    def get_manga_name(self) -> str:
        return self._get_name('/g/([^/]+/[^/?]+)').replace('/', '-')

    def prepare_cookies(self):
        self.helper = e_hentai_org.EHentaiOrg(self)

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        max_idx = self.helper.get_pages_count(parser)
        return list(range(max_idx, -1, -1))

    def get_files(self):
        url = self.helper.get_url() + '?p='
        select = '#gdt .gdtm > div > a'

        idx = self.chapter
        if idx == 0:
            content = self.content
        else:
            content = self.http_get('{}{}'.format(url, idx))
        return self.document_fromstring(content, select)

    def get_cover(self) -> str:
        return self._cover_from_content('#gd1 > div')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = EHentaiOrg



from zipfile import ZipFile, ZIP_DEFLATED

from PIL import Image
from os.path import splitext

from manga_py.fs import is_file, make_dirs, basename, dirname, unlink


class Archive:
    _archive = None
    files = None
    _writes = None

    def __init__(self):
        self.files = []
        self._writes = {}

    @staticmethod
    def _check_ext(file, name):
        file_name, file_ext = splitext(name)
        if file_ext == '' or file_ext == '.':
            try:
                image = Image.open(file)
                name = file_name.rstrip('.') + '.' + image.format.lower()
                image.close()
            except (FileNotFoundError, OSError):
                pass
        return name

    def write_file(self, data, in_arc_name):
        self._writes[in_arc_name] = data

    def add_file(self, file, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = basename(file)
        in_arc_name = self._check_ext(file, in_arc_name)
        self.files.append((file, in_arc_name))

    def set_files_list(self, files):
        self.files = files

    def add_book_info(self, data):
        self.write_file('comicbook.xml', data)

    def __add_files(self):
        for file in self.files:
            if is_file(file[0]):
                self._archive.write(file[0], file[1])

    def __add_writes(self):
        for file in self._writes:
            self._archive.writestr(file, self._writes[file])

    def add_info(self, data):
        self.write_file(data, 'info.txt')

    def make(self, dst):
        if not len(self.files) and not len(self._writes):
            return

        make_dirs(dirname(dst))

        self._archive = ZipFile(dst, 'w', ZIP_DEFLATED)
        self.__add_files()
        self.__add_writes()
        self._archive.close()
        self._maked()

    def _maked(self):
        for file in self.files:
            unlink(file[0])



from .pzykosis666hfansub_com import Pzykosis666HFansubCom


class AdultoSeinagiOrg(Pzykosis666HFansubCom):
    _name_re = '[^/]/[^/]+/([^/]+)/'
    _content_str = '{}/series/{}/'


main = AdultoSeinagiOrg



from .pzykosis666hfansub_com import Pzykosis666HFansubCom


class TripleSevenScansCom(Pzykosis666HFansubCom):
    _name_re = '/reader/[^/]+/([^/]+)/'
    _content_str = '{}/reader/series/{}/'


main = TripleSevenScansCom



from .pzykosis666hfansub_com import Pzykosis666HFansubCom


class BnsShounenAiNet(Pzykosis666HFansubCom):
    _name_re = '/read/[^/]+/([^/]+)/'
    _content_str = '{}/read/series/{}/'


main = BnsShounenAiNet



import json
import re
from abc import ABCMeta

from .base_classes import (
    Abstract,
    Archive,
    Base,
    Callbacks,
    # TODO
    CloudFlareProtect,
    Static
)
from .fs import (
    get_temp_path,
    is_file,
    basename,
    remove_file_query_params,
    path_join,
)
from .http import MultiThreads
from .meta import __downloader_uri__
from .meta import __version__
from .info import Info


class Provider(Base, Abstract, Static, Callbacks, metaclass=ABCMeta):
    _volumes_count = 0
    _archive = None
    _zero_fill = False
    _info = None
    _simulate = False
    _volume = None

    def __init__(self, info: Info = None):
        super().__init__()
        self.re = re
        self.json = json
        self._params['temp_directory'] = get_temp_path()
        self._info = info

    def _params_parser(self, params):
        # image params
        self._set_if_not_none(self._image_params, 'crop_blank', params.get('crop_blank', False))
        self._set_if_not_none(
            self._image_params, 'crop',
            (params.get('xt', 0),
             params.get('xr', 0),
             params.get('xb', 0),
             params.get('xl', 0))
        )
        # downloading params
        self._set_if_not_none(self._params, 'destination', params.get('destination', None))
        self._zero_fill = params.get('zero_fill')
        self._simulate = params.get('simulate')

    def process(self, url, params=None):  # Main method
        self._params['url'] = url
        params = params if isinstance(params, dict) else {}
        self._params_parser(params)
        for i in params:
            self._params.setdefault(i, params[i])

        self.prepare_cookies()
        self._storage['manga_name'] = self.get_manga_name()
        self._storage['main_content'] = self.get_main_content()
        self._storage['chapters'] = self._prepare_chapters(self.get_chapters())

        if not self._params.get('reverse_downloading', False):
            self._storage['chapters'] = self._storage['chapters'][::-1]

        self._storage['init_cookies'] = self._storage['cookies']
        self._info and self._info.set_ua(self.http().user_agent)

        self.loop_chapters()

    def _check_archive(self):
        # check
        _path = self.get_archive_path()
        not_allow_archive = not self._params.get('rewrite_exists_archives', False)

        return not_allow_archive and is_file(_path)

    def loop_chapters(self):
        volumes = self._storage['chapters']
        _min = self._params.get('skip_volumes', 0)
        _max = self._params.get('max_volumes', 0)
        if _max > 0:
            _max += _min - 1
        for idx, __url in enumerate(volumes):
            self._storage['current_chapter'] = idx

            self._info.add_volume(self.chapter, self.get_archive_path())

            if idx < _min or (idx > _max > 0) or self._check_archive():
                continue

            if not self._simulate:
                self.loop_callback_chapters()

                self._storage['files'] = self.get_files()
                self.loop_files()

    def loop_files(self):
        if isinstance(self._storage['files'], list) and len(self._storage['files']) > 0:
            self._archive = Archive()
            self._call_files_progress_callback()

            if self._params.get('no_multi_threads', False):
                self._one_thread_save(self._storage['files'])

            else:
                self._multi_thread_save(self._storage['files'])

            self.make_archive()

    def _save_file_params_helper(self, url, idx):
        if url is None:
            _url = self.http().normalize_uri(self.get_current_file())
        else:
            _url = url

        url = self.before_file_save(url, idx)

        filename = remove_file_query_params(basename(_url))
        _path = self.remove_not_ascii(self._image_name(idx, filename))
        _path = get_temp_path(_path)

        return _path, idx, url

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        _path, idx, _url = self._save_file_params_helper(url, idx)

        if not is_file(_path):
            self.http().download_file(_url, _path)
            self._archive.add_file(_path, in_arc_name)
        callable(callback) and callback()
        self.after_file_save(_path, idx)

        return _path

    def get_archive_path(self):
        _path = remove_file_query_params(self.get_archive_name())
        _path = self.remove_not_ascii(_path)

        if not _path:
            _path = str(self._storage['current_chapter'])

        name = self._params.get('name', '')
        if not len(name):
            name = self._storage['manga_name']

        return path_join(
            self._params.get('destination', 'Manga'),
            name,
            _path + '.%s' % self._archive_type()
        )

    def make_archive(self):
        _path = self.get_archive_path()

        info = 'Site: {}\nDownloader: {}\nVersion: {}'.format(self.get_url(), __downloader_uri__, __version__)

        # """
        # make book info
        # """
        # if self._params['cbz']:
        #     self._archive.add_book_info(self._arc_meta_info())

        self._archive.add_info(info)
        self._archive.make(_path)

    def html_fromstring(self, url, selector: str = None, idx: int = None):
        params = {}
        if isinstance(url, dict):
            params = url['params']
            url = url['url']
        return self.document_fromstring(self.http_get(url, **params), selector, idx)

    def _multi_thread_callback(self):
        self._call_files_progress_callback()
        self._storage['current_file'] += 1

    def _multi_thread_save(self, files):
        threading = MultiThreads()
        # hack
        self._storage['current_file'] = 0
        for idx, url in enumerate(files):
            threading.add(self.save_file, (idx, self._multi_thread_callback, url))

        threading.start()

    def _one_thread_save(self, files):

        for idx, __url in enumerate(files):
            self._storage['current_file'] = idx
            self._call_files_progress_callback()
            self.save_file()

    def cf_protect(self, url):
        """
        WARNING! Thins function replace cookies!
        :param url: str
        :return:
        """
        cf = CloudFlareProtect()
        params = cf.run(url)
        if len(params):
            self._storage['cookies'] = params[0]
            self.update_ua(params[1])
            self._params['cf-protect'] = True

    def update_ua(self, ua):
        self._storage['user_agent'] = ua
        self.http().user_agent = ua
        self._info and self._info.set_ua(ua)



from manga_py.crypt import BaseLib
from manga_py.provider import Provider
from .helpers.std import Std


class ShogakukanTameshiyoMe(Provider, Std):
    __local_storage = None
    _site = 'https://shogakukan.tameshiyo.me'
    img_url = '/imgDeliver?gcode='

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        return self._get_name('/(\d+)')

    def get_chapters(self):
        return [b'']

    def before_file_save(self, url, idx):
        _url = self._site + self.img_url + self.__local_storage['code']
        _url = self.http_post(_url, data={
            'base64': '1',
            'vsid': self.__local_storage['vsid'],
            'trgCode': url,
        }, headers={'Referer': self._site})
        return _url

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        if url is None:
            _url = self.get_current_file()
        else:
            _url = url

        _path, idx, _url = self._save_file_params_helper(_url, idx)
        _path += '.jpg'

        with open(_path, 'wb') as file:
            file.write(BaseLib.base64decode(_url))

        self._archive.add_file(_path, in_arc_name)
        callable(callback) and callback()

        return _path

    @property
    def chapter_url(self):
        url = self.chapter
        if isinstance(url, bytes):
            url = self.get_url()
        return url

    def get_files(self):
        parser = self.html_fromstring(self.chapter_url)
        self.__local_storage = {
            'vsid': parser.cssselect('input[data-key="vsid"]')[0].get('value'),
            'code': parser.cssselect('input[data-key="isbn1kan"]')[0].get('value'),
        }
        return [i.get('value') for i in parser.cssselect('input[data-key="imageCodes"]')]

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ShogakukanTameshiyoMe



from manga_py.provider import Provider
from .helpers.std import Std


class GoMangaCo(Provider, Std):
    _name_re = '/reader/[^/]+/([^/]+)/'
    _content_str = '{}/reader/series/{}/'
    _chapters_selector = '.list .element .title a'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-', 2)
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        url = self.chapter
        index_re = r'/rea\w+/[^/]+/[^/]+/(?:[^/]+/)?(\d+/\d+(?:/\d+)?)'
        group = self.re.search(index_re, url).group(1)
        return group.replace('/', '-')

    def get_main_content(self):
        return self._get_content(self._content_str)

    def get_manga_name(self) -> str:
        return self._get_name(self._name_re)

    def get_chapters(self):
        return self._elements(self._chapters_selector)

    def _get_json_selector(self, content):
        idx = self.re.search(r'page_width\s=\sparseInt\((\w+)\[', content).group(1)
        return r'var\s{}\s*=\s*(\[.+\])'.format(idx)

    def get_files(self):
        content = self.http_get(self.chapter)
        selector = self._get_json_selector(content)
        items = self.json.loads(self.re.search(selector, content).group(1))
        return [i.get('url') for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.thumbnail img')

    def prepare_cookies(self):
        url = self.get_url()
        self.cf_protect(url)
        data = {'adult': 'true'}
        try:
            response = self.http().requests(method='post', data=data, url=url)
            cookies = response.cookies.items()
            for i in cookies:
                self._storage['cookies'][i[0]] = i[1]
        except Exception:
            pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = GoMangaCo



import tempfile
from os import path, name as os_name, getpid, unlink as os_unlink, makedirs, stat
from pathlib import Path
from shutil import rmtree, move

__dir_name__ = '.PyMangaDownloader'


def get_temp_path(*args):
    temp = 'temp_%s' % getpid()
    return path.join(tempfile.gettempdir(), __dir_name__, temp, *args)


def root_path():
    return path.dirname(path.dirname(path.realpath(__file__)))


def get_util_home_path():
    if os_name == 'nt':
        home = path.join(str(Path.home()), 'AppData', 'Roaming', __dir_name__)
    else:
        home = path.join(str(Path.home()), __dir_name__)
    return home


def make_dirs(directory):
    path.isdir(directory) or makedirs(directory)


def remove_file_query_params(name, save_path: bool = True) -> str:
    file_path = path.dirname(name)
    name = path.basename(name)
    position = name.find('?')
    if position == 0:
        name = 'image.png'  # fake image name
    elif position > 0:
        name = name[:position]
    return path.join(file_path, name) if save_path else name


def is_file(_path):
    return path.isfile(_path)


def is_dir(_path):
    return path.isdir(_path)


def basename(_path):
    return path.basename(_path)


def dirname(_path):
    return path.dirname(_path)


def path_join(_path, *args):
    return path.join(_path, *args)


def unlink(_path):
    if is_dir(_path):
        return rmtree(_path)
    if is_file(_path):
        return os_unlink(_path)


def os_stat(_path):
    if is_file(_path):
        return stat(_path)
    return None


def file_size(_path):
    data = os_stat(_path)
    if data:
        return data.st_size
    return None


def rename(_from, _to):
    if is_file(_from) or is_dir(_from):
        is_dir(dirname(_to)) or makedirs(dirname(_to))
        move(_from, _to)


def storage(_path):
    _path = get_temp_path('storage', _path)
    make_dirs(dirname(_path))
    return _path



from manga_py.provider import Provider
from .helpers.std import Std


class HeavenMangaBiz(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-', 2)
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        return self.re.search(r'-chap-(\d+)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        s = self.domain[self.domain.rfind('.'):]
        selector = r'\%s/([^/]+)'
        if ~self.get_url().find('-chap-'):
            selector += '-chap-'
        return self._get_name(selector % s)

    def get_chapters(self):
        selector = '.chapters-wrapper h2.chap > a'
        pages = self._elements('a.next.page-numbers')
        items = self._elements(selector)
        if pages:
            pages = self.re.search(r'/page-(\d+)', pages[-1].get('href')).group(1)
            for i in range(1, int(pages)):
                url = '{}/{}/page-{}'.format(self.domain, self.manga_name, i + 1)
                items += self._elements(selector, self.http_get(url))
        return items

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.chapter-content img')

    def get_cover(self) -> str:
        return self._cover_from_content('.comic-info .thumb > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = HeavenMangaBiz



from manga_py.crypt import MangaRockComCrypt
from manga_py.fs import rename, unlink, basename
from manga_py.provider import Provider
from .helpers.std import Std


class MangaRockCom(Provider, Std):
    crypt = None
    __api_uri = 'https://api.mangarockhd.com/query/web400/'

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)-\d+')

    def get_chapters(self):
        idx = self._get_name('/manga/([^/]+)')
        url = '{}info?oid={}&last=0&country=Japan'.format(self.__api_uri, idx)
        items = self.json.loads(self.http_get(url))
        return [i.get('oid').encode() for i in items.get('data', {}).get('chapters', [])]

    def get_files(self):
        url = '{}pages?oid={}&country=Japan'.format(self.__api_uri, self.chapter.decode())
        items = self.json.loads(self.http_get(url))
        return items.get('data')

    # decrypt
    def after_file_save(self, _path, idx: int):
        _path_wp = _path + 'wp'
        file_r = open(_path, 'rb')
        file_w = open(_path_wp, 'wb')
        file_w.write(self.crypt.decrypt(file_r.read()))
        file_r.close()
        file_w.close()
        unlink(_path)
        rename(_path_wp, _path)

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        _path, idx, _url = self._save_file_params_helper(url, idx)
        in_arc_name = basename(_path) + '.webp'
        return super().save_file(idx, callback, _url, in_arc_name)

    def get_cover(self) -> str:
        selector = 'div:not([class]) > div[class] > div[class] > div[class] > div[class] > img'
        url = '{}{}'.format(self.domain, self._get_name('(/manga/[^/]+)'))
        img = self._elements(selector, self.http_get(url))
        if img and len(img):
            return img[0].get('src')

    def prepare_cookies(self):
        self.crypt = MangaRockComCrypt()

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaRockCom



from manga_py.provider import Provider
from .helpers.std import Std


class TranslateWebToonsCom(Provider, Std):
    _content = None

    def get_archive_name(self) -> str:
        return self.normal_arc_name([self.get_chapter_index()])

    def get_chapter_index(self) -> str:
        return self.re.search(r'\bepisodeNo=(\d+)', self.chapter).group(1)

    def get_main_content(self):
        if self._content is None:
            self._content = self.http_get(self.get_url())
        return self._content

    def get_manga_name(self) -> str:
        return self.text_content(self.content, 'h3.subj')

    def _chapters(self, content):
        return self._elements('.detail_lst > ul > li > a', content)

    def _filter_chapters(self, chapters):
        result = []
        for item in chapters:
            content = item.cssselect('.rate_num.cplt')[0].text_content().strip('\n\t\r \0')
            if content == '100%':
                result.append(item)
        return result

    def get_chapters(self):
        pages = self._elements('.paginate > a:not([class])')
        chapters = self._chapters(self.content)
        if pages:
            n = self.http().normalize_uri
            for i in pages:
                content = self.http_get(n(i.get('href')))
                chapters += self._chapters(content)
        return self._filter_chapters(chapters)

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.img_info > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.thmb img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TranslateWebToonsCom



import html

from manga_py.provider import Provider
from .helpers.std import Std


class MangaHubRu(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/read/[^/]+/[^\d]+(\d+)/(\d+)/', self.chapter).groups()
        return '{}-{}'.format(*idx)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self.re.search(r'\.ru/([^/]+)/?', self.get_url())

    def get_chapters(self):
        return self.document_fromstring(self.content, '.b-catalog-list__name a[href^="/"]')

    def get_files(self):
        parser = self.html_fromstring(self.chapter, '.b-main-container .b-reader__full')
        if not parser:
            return []
        result = parser[0].get('data-js-scans')
        result = self.json.loads(html.unescape(result.replace('\/', '/')))
        domain = self.domain
        return [domain + i['src'] for i in result]

    def get_cover(self):
        return self._cover_from_content('.manga-section-image__img img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHubRu



from time import sleep

from manga_py.fs import get_temp_path, make_dirs, remove_file_query_params, basename, path_join, dirname
from .multi_threads import MultiThreads
from .request import Request
from .url_normalizer import normalize_uri


class Http(Request):
    count_retries = 20

    def __init__(
            self,
            allow_webp=True,
            referer='',
            user_agent=None,
            proxies=None,
            cookies=None,
            kwargs=None
    ):
        super().__init__()
        self.__set_param('allow_webp', allow_webp)
        self.__set_param('referer', referer)
        self.__set_param('user_agent', user_agent)
        self.__set_param('proxies', proxies)
        self.__set_param('cookies', cookies)
        self.__set_param('kwargs', kwargs)

    def __set_param(self, name, value):
        if value is not None:
            self_val = getattr(self, name)
            _type = type(self_val)
            if self_val is not None and not isinstance(value, _type):
                raise AttributeError('{} type not {}'.format(name, _type))
            setattr(self, name, value)

    def __download(self, file_name, url, method):
            while True:
                with open(file_name, 'wb') as out_file:
                    response = self.requests(url, method=method, timeout=60, allow_redirects=True)
                    if response.status_code >= 400:
                        sleep(.25)
                        continue
                    out_file.write(response.content)
                    response.close()
                    out_file.close()
                    break

    def _safe_downloader(self, url, file_name, method='get') -> bool:
        try:
            make_dirs(dirname(file_name))
            url = self.normalize_uri(url)
            self.__download(file_name, url, method)
        except OSError as ex:
            self.debug and print(ex)
            return False
        return True

    def _download_one_file_helper(self, url, dst, callback: callable = None):
        r = 0
        while r < self.count_retries:
            if self._safe_downloader(url, dst):
                return True

            r += 1
            mode = 'Retry'
            if r >= self.count_retries:
                mode = 'Skip image'
            callable(callback) and callback(text=mode)
        return False

    def download_file(self, url: str, dst: str = None, callback: callable = None) -> bool:
        if not dst:
            name = basename(remove_file_query_params(url))
            dst = path_join(get_temp_path(), name)
        return self._download_one_file_helper(url, dst, callback)

    def normalize_uri(self, uri, referer=None):
        if not referer:
            referer = self.referer
        if isinstance(uri, str):
            return normalize_uri(uri.strip(), referer)
        return uri

    def multi_download_get(self, urls, dst: str = None, callback: callable = None):
        threading = MultiThreads()
        for url in urls:
            threading.add(self.download_file, (url, dst))
        threading.start(callback)

    def get_redirect_url(self, url, **kwargs):
        location = self.requests(url=url, method='head', **kwargs)
        url = location.headers.get('Location', url)
        return self.normalize_uri(url)



from .komikid_com import KomikIdCom


class NozomiNoFansubCom(KomikIdCom):
    _content_str = '{}/public/manga/{}'

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/[^/]+/.+?(\d+(?:[^\d/]\d+)?)')
        ch = self.chapter
        return re.search(ch).group(1)


main = NozomiNoFansubCom
